% Frequency Response of Simple Quarter Car Model with Standard and Sky Hook Damper
clear all, close all

global  sky_hook
global  mass_c  mass_w  c_susp  d_susp  d_c d_w t_d  c_tire
global  g_const f0_susp  f0_tire  ftire  zcpp 
global  q  p  amp_0

% vehicle data
g_const = 9.81; % Constant of Gravity [m/s^2]
mass_c=1200/4;  % Chassis Mass (Quarter Car) [kg]
mass_w=50;      % Mass of Knuckle and Wheel [kg]
c_susp=18000;   % Suspension Stiffness [N/m]
d_susp= 3350;   % Standard Suspension Damping [N/(m/s)]
d_c = 3600;     % Sky Hook Chassis
d_w = 1450;     % Sky Hook Wheel
t_d=0.02;       % Time constant for dynamic damping
c_tire=220000;  % Tire Stiffness [N/m]

% define sweep sine excitation
f0   =  0.2;    % lowest frequency [Hz]
fe   =   30;    % highest frequency [Hz]
n    =   50;    % number of frequency interval
amp_0=  0.02;   % amplitudes [m]

% sweep sine parameter
q = 1/n * log ( fe/f0 ) ;   p = q / ( f0 * ( 1 - (f0/fe)^(1/n) ) ) ;

% Damper type
sky_hook = menu('sky-hook:','no','active','passive') - 1;

% constants
f0_susp = mass_c*g_const;
f0_tire = f0_susp + mass_w*g_const;

% initial values
t0 = 0;  x0 = [ 0; 0; 0; 0; d_susp ];

disp(' '), disp('quarter car model')
disp(' '), disp(['frequency response:  A = ',num2str(amp_0)])
disp(' ')

% pre-allocate vars to speed up loop
freq=zeros(n+1,1); y0=freq; zbs=freq; zbc=freq; bas=freq; bac=freq; whls=freq; whlc=freq;

% perform time simulation over corresponding period
for ifreq=1:n+1

% Interval from t0 to te results in frequency freq = 1/(te-t0)
  te=p/q*(1-exp(-ifreq*q));  freq(ifreq) = 1 / ( te - t0 ) ;
  disp([' frequency  f=', num2str( freq(ifreq)) ])
  [t,x] = ode45(@qcm_simple_freq_response_f,[t0,te],x0) ;
  
% comute excitation signal
  arg = 2*pi/q*log(p./(p-q*t)); se=sin(arg); ce=cos(arg);
  
% curve fit of different signals to 1. harmonic:  y = y0 + ys*sin(arg) + yc*cos(arg)
  A = [ones(size(arg))  se  ce  ];  
  
% body displacement
  zb = x(:,1)/amp_0; 
  coeff = A \ zb ;         
  y0(ifreq) = coeff(1);  % offset
  zbs(ifreq) = coeff(2);  % sine part
  zbc(ifreq) = coeff(3);  % cosine part

% body acceleration and wheel load
  ba = zb; whl= zb;
  for i=1:length(t)
	xp = qcm_simple_freq_response_f(t(i),x(i,:)');
	ba(i) = zcpp/g_const;
	whl(i)= (ftire-f0_tire)/f0_tire;
  end
  coeff = A \ ba ;         
  y0(ifreq)  = coeff(1);   % offset
  bas(ifreq) = coeff(2);  % sine part
  bac(ifreq) = coeff(3);  % cosine part
  
  coeff = A \ whl ;         
  y0(ifreq) = coeff(1);   % offset
  whls(ifreq) = coeff(2);  % sine part
  whlc(ifreq) = coeff(3);  % cosine part

% set initial values for next interval
  t0=te;  x0=x(length(t),:) ;

end

% amplitude and phase
amp_b  = sqrt(zbs.^2+zbc.^2) ;
acc_b  = sqrt(bas.^2+bac.^2) ;
whl_b  = sqrt(whls.^2+whlc.^2);

phi_b  = atan2(zbc,zbs)*180/pi ;

% plot results

hf=figure;
switch sky_hook
    case 0, set(hf,'Name','Standard Damper')
    case 1, set(hf,'Name','Skyhook active')
    case 2, set(hf,'Name','Skyhook passive')
end

subplot(3,1,1), hold on
semilogx(freq,amp_b,'Linewidth',2), grid on
title(['body displacement at A = ' num2str(amp_0*1000) ' mm'])
xy=axis; axis([0,fe,xy(3),xy(4)])

subplot(3,1,2), hold on
semilogx(freq,acc_b,'Linewidth',2), grid on
title(['body acceleration at A = ' num2str(amp_0*1000) ' mm'])
xy=axis; axis([0,fe,xy(3),xy(4)])

subplot(3,1,3), hold on
semilogx(freq,whl_b,'Linewidth',2), grid on
title(['wheel load at A = ' num2str(amp_0*1000) ' mm'])
xy=axis; axis([0,fe,xy(3),xy(4)])