% Steady State Brake and Drive Pitch 
clear all, close all

% Vehicle data (Planar Model)
g  = 9.81;            % Constant of Gravity
R  = 0.30;            % Tire Radius [m]
h  = 0.59;            % Height of Center of Gravity [m]
a1 = 1.20;            % Chassis Center of Gravity --> Front Axle [m]
a2 = 1.30;            % Chassis Center of Gravity --> Rear Axle [m]
mC  = 1200;           % Mass of Chassis [kg]
m1  = 2*45;           % Mass of Front Axle [kg]
m2  = 2*45;           % Mass of Rear Axle [kg]
ThW1 = 2*1.2;         % Inertia of Front Wheels [kg m^2]
ThW2 = 2*1.2;         % Inertia of Rear Wheels [kg m^2]
cS1 = 2*20000;        % Stiffness of Front Axle Suspension [N/m]
cS2 = 2*25000;        % Stiffness of Rear Axle Suspension [N/m]
cT1 = 2*220000;       % Stiffness of Front Wheels [N/m]
cT2 = 2*200000;       % Stiffness of Rear Wheels [N/m]

% Axle Kinematics
dx1 = tan(  0/180*pi);  % Inclination of Front Wheel Motion [-]
dx2 = tan(  0/180*pi);  % Inclination of Rear Wheel Motion [-]
db1 =       0/180*pi;   % y-Rotation of Front Wheel Body [rad/m]
db2 =       0/180*pi;   % y-Rotation of Rear Wheel Body [rad/m]

% Constants
mV  = mC + m1 + m2 ;      % overall vehicle mass
hV  = ( m1*R + m2*R + mC*h ) / mV;  % height of COG vehicle
a1V = ( a1*mC + (a1+a2)*m2 ) / mV;  % vehicle COG --> front axle [m]
a2V = ( a2*mC + (a1+a2)*m1 ) / mV;  % vehicle COG --> rear axle [m]

% Driving or Braking 
vp_g = input('normalized vehicle acceleration/deceleration = ');

% use simple planar model to estimate appropriate torques by neglecting wheel inertias  
if vp_g < 0 
    k_opt = (a1V+hV*vp_g)/(a2V-hV*vp_g); % optimal brake force distribution 
    FB = (mV+ThW1/R^2+ThW2/R^2)*g*vp_g/(1+k_opt) ; %   Brake Force 
    TB1 = -FB*R;   TB2 = -k_opt*FB*R; % braking torques
    TD1 = 0;  TD2 = 0; % no driving torques
else
    f_opt = (a2V-hV*vp_g)/(a1+a2);  % optimal drive force distribution
    FD = (mV+ThW1/R^2+ThW2/R^2)*g*vp_g ;  % driving force 
    TD1 = f_opt*FD*R ;   TD2 = (1-f_opt)*FD*R ;  % driving torques
    TB1 = 0;  TB2 = 0;  % no braking torques
end   

% overall and generalized vehicle mass
mV=mC+m1+m2; mg = mV+ThW1/R^2+ThW2/R^2;

% static wheel loads
Fz1_st = a2/(a1+a2)*mC*g + m1*g;  Fz2_st = a1/(a1+a2)*mC*g + m2*g;

% vehicle acceleration
xddot = 1/R*(TD1+TD2-TB1-TB2)/mg;

% dynamic tire forces
Fx1=(TD1-TB1)/R - ThW1/R^2*xddot;
Fx2=(TD2-TB2)/R - ThW2/R^2*xddot;
Fz1=-xddot*((ThW1+ThW2)/R+(m1+m2)*R+mC*h)/(a1+a2);
Fz2= xddot*((ThW1+ThW2)/(R*h)+(m1+m2)*R/h+mC)*h/(a1+a2);

% dynamic suspension forces 
FS1=Fz1+(dx1-R*db1)*Fx1+db1*TD1-(dx1*m1+db1*ThW1/R)*xddot;
FS2=Fz2+(dx2-R*db2)*Fx2+db2*TD2-(dx2*m2+db2*ThW2/R)*xddot;

% suspension travel
z1 = FS1/cS1;  z2=FS2/cS2;  

% pitch and hub motion of chassis
bC = (Fz1/cT1-Fz2/cT2+z1-z2)/(a1+a2);   zC = a1*bC-z1-Fz1/cT1;    

% Needed Friction Coefficients
mue1 = abs(Fx1)/(Fz1_st+Fz1);  mue2 = abs(Fx2)/(Fz2_st+Fz2);

disp('----------------------------------------------------------------')
disp(' ')
disp(['Drive Torque at Front Axle:        ',num2str(TD1),' Nm'])
disp(['Drive Torque at Rear Axle:         ',num2str(TD2),' Nm'])
disp(['Brake Torque at Front Axle:        ',num2str(TB1),' Nm'])
disp(['Brake Torque at Rear Axle:         ',num2str(TB2),' Nm'])
disp(' ')
disp(['Inclination at Front Axle:         ',num2str(atan(dx1)*180/pi),' deg'])
disp(['Inclination at Rear Axle:          ',num2str(atan(dx2)*180/pi),' deg'])
disp(['y-Rotation at Front Axle:          ',num2str(db1*180/pi),' deg/m'])
disp(['y-Rotation at Rear Axle:           ',num2str(db2*180/pi),' deg/m'])
disp(' ')
disp(['Vehicle Acceleration:              ',num2str(xddot/g),' g'])
disp(['long. force at front axle:         ',num2str(Fx1),', N'])
disp(['long. force at rear axle:          ',num2str(Fx2),', N'])
disp(['dyn. wheel at load front axle:     ',num2str(Fz1),', N'])
disp(['dyn. wheel at load rear axle:      ',num2str(Fz2),', N'])
disp(['dyn. susp. force at front axle:    ',num2str(FS1),', N'])
disp(['dyn. susp. force at rear axle:     ',num2str(FS2),', N'])
disp(' ')
disp(['Suspension Travel Front:           ',num2str(z1*1000),' mm'])
disp(['Suspension Travel Rear:            ',num2str(z2*1000),' mm'])
disp(['Hub-Motion Chassis:                ',num2str(zC*1000),' mm'])
disp(['Pitch-Motion Chassis               ',num2str(bC*180/pi),' deg'])
disp(' ')
disp(['Axle Load Front (static+dynamic):  ',num2str(Fz1_st+Fz1)])
disp(['Axle Load Rear (static+dynamic):   ',num2str(Fz2_st+Fz2)])
disp(' ')
disp(['Needed Friction Coefficient Front: ',num2str(mue1)])
disp(['Needed Friction Coefficient Rear:  ',num2str(mue2)])
disp(' ')
disp('----------------------------------------------------------------')


% a n i m a t e   r e s u l t s

if vp_g > 0
    titel='Pitch reaction to driving';
else
    titel='Pitch reaction to braking';
end
hf=figure;
set(hf,'Name',titel ...
      ,'NumberTitle','off' ...
      ,'MenuBar','none' ...
      ,'Color',[0.95 0.95 0.95] )
%
hold on
  ht=text(0,3.50*h,['a_x/g = ',num2str(vp_g)]);
  set(ht,'HorizontalAlignment','center'), set(ht,'FontSize',16)
  ht=text(0,3.00*h,['z_C = ',num2str(1000*zC),' [mm]']);
  set(ht,'HorizontalAlignment','center'), set(ht,'FontSize',16)
  ht=text(0,2.50*h,['\beta_C = ',num2str(bC*180/pi),' [^o]']);
  set(ht,'HorizontalAlignment','center'), set(ht,'FontSize',16)
  ht=text( a1,2.00*h,['z_1 = ',num2str(1000*z1),' [mm]']);
  set(ht,'HorizontalAlignment','center'), set(ht,'FontSize',16)
  ht=text(-a2,2.00*h,['z_2 = ',num2str(1000*z2),' [mm]']);
  set(ht,'HorizontalAlignment','center'), set(ht,'FontSize',16)
  ht=text( a1,-R,['\mu_1 = ',num2str(mue1)]);
  set(ht,'HorizontalAlignment','center'), set(ht,'FontSize',16)
  ht=text( a1,-2*R,'front');
  set(ht,'HorizontalAlignment','center'), set(ht,'FontSize',24)
  ht=text(-a2,-R,['\mu_2 = ',num2str(mue2)]);
  set(ht,'HorizontalAlignment','center'), set(ht,'FontSize',16)
  ht=text(-a2,-2*R,'rear');
  set(ht,'HorizontalAlignment','center'), set(ht,'FontSize',24)
  
% Vehicle Body
  x1 = 1.15*(a1+R);   x2 = 1.25*(a2+R);
  xa0 = [ -x2; -x2;  x1;  x1 ] ;
  za0 = [  -1;   1;   1;  -1 ] * (h-R) ;

% Wheels
  phi = [ 0:0.05:1 ]*2*pi;   xr0 = R*sin(phi);   zr0 = R*cos(phi);
  xr1 = a1 + xr0;   zr1 = R  + zr0;
  xr2 =-a2 + xr0;   zr2 = R  + zr0;

% Road
  plot([xa0(1); xa0(4)]*1.2,[0; 0],'k','Linewidth',3)

% Initial Configuration
  plot([xa0;xa0(1)],[za0;za0(1)]+h,'--k','Linewidth',1.5)
  plot(xr1,zr1,'k','Linewidth',2)
  plot(xr2,zr2,'k','Linewidth',2)
  plot(0,h,'xk','LineWidth',2,'Markersize',15)

% rotated body
  xa =  cos(bC).*xa0 + sin(bC).*za0;
  za = -sin(bC).*xa0 + cos(bC).*za0;
  plot([xa;xa(1)],[za;za(1)]+h+zC,'r','Linewidth',1.5)

% wheel center motion
  plot([ a1-R/2*dx1; a1+R/2*dx1],[R-R/2;R+R/2],'Color',[0,0,0.7],'Linewidth',1.5)
  plot([-a2-R/2*dx2;-a2+R/2*dx2],[R-R/2;R+R/2],'Color',[0,0,0.7],'Linewidth',1.5)

% contact point motion
  dxc1 = dx1 - R*db1;
  dxc2 = dx2 - R*db2;
  plot([ a1-R/2*dxc1; a1+R/2*dxc1],[-R/2;+R/2],'Color',[0,0.7,0],'Linewidth',1.5)
  plot([-a2-R/2*dxc2;-a2+R/2*dxc2],[-R/2;+R/2],'Color',[0,0.7,0],'Linewidth',1.5)

% brake pitch pole
  if vp_g < 0
    dx = (a1+a2)*0.8;
    plot([ a1-dx; a1   ],[ dx*dxc1;   0    ],'Color',[0,0.7,0],'Linewidth',1.5)
    plot([-a2;   -a2+dx],[   0    ;-dx*dxc2],'Color',[0,0.7,0],'Linewidth',1.5)
  end

axis equal, axis off