% Track Model Vehicle and Trailer or Tractor-Semitrailer
clear all, close all
%
global a b c v  t_d d_i

% vehicle data
a = 2.6;    % wheel base [m]
b = 1.0;    % distance rear axle --> coupling point [m]
c = 2.0;    % coupling point --> trailer axle [m]
v = 5./3.6; % driving velocity [km/h --> m/s]

% define lookup table for steer input [time, steering angle] 
t_d = [  0  20  30 ] ;
d_i = [  0  45  45 ]*pi/180 ; % deg --> rad

% get time interval
t0 = t_d(1);  te = t_d(length(t_d)) ;

% initial conditions (vehicle position and orientation)
x0 = 0;  y0 = 0;  ga0 = 0/180*pi ;  % yaw angle [deg --> rad]
% bending angle adjusted to steering angle (steady state solution)
d0 = interp1 ( t_d, d_i, t0 );
ka0= asin(-c*tan(d0)/sqrt((b*tan(d0))^2+a^2)) - atan2(b*tan(d0),a) ;

% time simulation 
[t,x] = ode45(@track_model_f, t0:1/25:te, [x0; y0; ga0;  ka0]);

% compute steering angle
d=interp1(t_d,d_i,t); 
% get yaw angle
ga=x(:,3);  
% bend angle
ka=x(:,4); 
% steady state bending angle
ka_st = asin(-c*tan(d)./sqrt((b*tan(d)).^2+a^2))-atan2(b*tan(d),a); 
% front axle
xa1 = x(:,1) + a*cos(ga);
ya1 = x(:,2) + a*sin(ga);
% rear axle
xa2 = x(:,1) ;
ya2 = x(:,2) ;
% coupling point and trailer axle
xk = x(:,1) - b*cos(ga) ;
yk = x(:,2) - b*sin(ga) ;
xa3 = xk - c*cos(ga+ka);
ya3 = yk - c*sin(ga+ka);

% plot some results
subplot(2,2,1)
plot(t,d*180/pi), title('\delta(t) [deg]'), grid on
subplot(2,2,2)
plot(t,v^2/a*tan(d)/9.81), title('a_y/g [-]'), grid on
subplot(2,2,3), hold on, grid on
plot(xa1,ya1,'k'), plot(xa2,ya2,'--b'),plot(xa3,ya3,'r'), title('path of axles')
axis equal, xlabel('[m]'), ylabel('[m]'), legend('Front','Rear','Trailer')
subplot(2,2,4) 
plot(t,[x(:,4),ka_st]*180/pi), title('\kappa(t) (dyn+st) [deg]'), grid on


k = menu('Animate results?','yes','no') ; 
if k==1  
  
  % Driving Area  
  x_1 = 1.05*min([min(xa1),min(xa2),min(xa3)]);
  x_2 = 1.05*max([max(xa1),max(xa2),max(xa3)]);
  y_1 = 1.05*min([min(ya1),min(ya2),min(ya3)]);
  y_2 = 1.05*max([max(ya1),max(ya2),max(ya3)]);

  b_min = min(x_1,y_1);  b_max = max(x_2,y_2);

  % fictitious pivot centers of axles
  xc1 = xa1 - sin(ga+d)*3*a;
  yc1 = ya1 + cos(ga+d)*3*a;
  xc2 = xa2 - sin(ga)*3*a;
  yc2 = ya2 + cos(ga)*3*a;
  xc3 = xa3 - sin(ga+ka)*3*a;
  yc3 = ya3 + cos(ga+ka)*3*a;

  h_fig=figure;
  set(h_fig,'DoubleBuffer','on' ...
           ,'Name','Animation' ...
           ,'NumberTitle','off' ...
           ,'MenuBar','none' ...
           ,'Color',[0.9 0.9 0.9] )
  
  % plot rectangular frame to get size of axis       
  plot([x_1,x_2],[y_1,y_1],'k'), hold on
  plot([x_1,x_2],[y_2,y_2],'k'), hold on
  plot([x_1,x_1],[y_1,y_2],'k'), hold on
  plot([x_2,x_2],[y_1,y_2],'k'), hold off
  axis equal, ax = axis;

  for i=1:length(t)
    if b>0
      plot([xa1(i),xk(i)],[ya1(i),yk(i)],'k','Linewidth',2);
    else
      plot([xa1(i),xa2(i)],[ya1(i),ya2(i)],'k','Linewidth',2);
    end   
    hold on
    plot([xk(i),xa3(i)],[yk(i),ya3(i)],'r','Linewidth',3);
    plot([xa1(i),xc1(i)],[ya1(i),yc1(i)],'b','Linewidth',1);
    plot([xa2(i),xc2(i)],[ya2(i),yc2(i)],'b','Linewidth',1);
    plot([xa3(i),xc3(i)],[ya3(i),yc3(i)],'b','Linewidth',1);
    plot(xa1(1:i),ya1(1:i),'k')
    plot(xa2(1:i),ya2(1:i),'k')
    plot(xa3(1:i),ya3(1:i),'k')
    axis(ax), grid on
    drawnow, hold off
  end

end

