% steady state tire forces
clear all, close all

% tire characteristics in longitudinal and lateral direction
dfx0 =  69000;    dfy0 =  66000;   % init slopes dfx/dsx & dfydsy [n/-]
fxm  =   3100;    fym  =   2950;   % maximum forces [n]
sxm  =  0.160;    sym  =  0.205;   % s where f(s) = fm [-]
fxs  =   2800;    fys  =   2800;   % sliding forces [n]
sxs  =  0.500;    sys  =  0.500;   % s where f(s) = fs [-]

% slip normalizing factors
hsxn = sxm/(sxm+sym) + (fxm/dfx0)/(fxm/dfx0+fym/dfy0);
hsyn = sym/(sxm+sym) + (fym/dfy0)/(fxm/dfx0+fym/dfy0);

% set slip values
nvar = 201;  sx = linspace(-1,1,nvar)*1.2*max(sxs);  sy = sx; 
% pre-allocate fx and fy to speed up loop
fx = zeros(length(sx),length(sy)); fy=zeros(length(sx),length(sy));

for i = 1:length(sx)
  for j = 1:length(sy) 
%   combined slip
    sxn = sx(i)/hsxn; syn = sy(j)/hsyn;  sc = sqrt ( sxn^2 + syn^2 );
    if sc>0, cphi=sxn/sc;sphi=syn/sc; else cphi=sqrt(2)/2;sphi=sqrt(2)/2; end
%   combined characteristic for normalized slip values
    df0 = sqrt( (dfx0*hsxn*cphi)^2 + (dfy0*hsyn*sphi)^2 );
    fm  = sqrt( (fxm*cphi)^2       + (fym*sphi)^2       );
    sm  = sqrt( (sxm/hsxn*cphi)^2  + (sym/hsyn*sphi)^2  );
    fs  = sqrt( (fxs*cphi)^2       + (fys*sphi)^2       );
    ss  = sqrt( (sxs/hsxn*cphi)^2  + (sys/hsyn*sphi)^2  );
%   combined tire force 
    [ f, fos ] = tmy_fcombined( sc, df0,fm,sm,fs,ss );  
%   longitudinal and lateral forces
    fx(i,j) = f*cphi;  fy(i,j) = f*sphi;    
  end
end

subplot(2,2,1), is=101:20:201; % selected lateral slips only
plot(sx,fx(:,is)/1000), axis([-0.6,0.6,-4,4]), grid on
title('f_x=f_x(s_x,s_y)'), xlabel('s_x'), ylabel('kN')
for k=1:length(is), txt(k)=cellstr(['sy=',num2str(sy(is(k)))]); end
legend(txt,'location','northwest'), clear txt

subplot(2,2,2), is=111:10:151; % selected longitudinal slips only
plot(sy,fx(is,:)'/1000), axis([-0.6,0.6,0,4]), grid on
title('f_x=f_x(s_x,s_y)'), xlabel('s_y'), ylabel('kn')
for k=1:length(is), txt(k)=cellstr(['sx=',num2str(sx(is(k)))]); end
legend(txt,'location','northwest'), clear txt

subplot(2,2,3), is=111:10:151; % selected lateral slips only
plot(sx,fy(:,is)/1000), axis([-0.6,0.6,0,4]), grid on
title('f_y=f_x(s_x,s_y)'), xlabel('s_x'), ylabel('kn')
for k=1:length(is), txt(k)=cellstr(['sy=',num2str(sy(is(k)))]); end
legend(txt,'location','northwest'), clear txt

subplot(2,2,4), is=101:20:201; % selected longitudinal slips only
plot(sy,fy(is,:)'/1000), axis([-0.6,0.6,-4,4]), grid on
title('f_y=f_x(s_x,s_y)'), xlabel('s_y'), ylabel('kn')
for k=1:length(is), txt(k)=cellstr(['sx=',num2str(sx(is(k)))]); end
legend(txt,'location','northwest'), clear txt


% e x t e n s i o n   t o   c a m b e r - i n f l u e n c e 

camber = [ 0, 2, 4]*pi/180; % different camber angles [deg -> rad]
vt=20; rd=0.310;  omega = vt/rd;  % set angular velocity (omega=vt/rd)

% tire offset data
nto0=0.178;    % tire offset @ sy = 0
synto0=0.20;   % sy where nto passes sy-axis
syntos=0.35;   % sy where nto approaches sy-axis again

% tire bore radius  (estimated from length l and width b of contact patch)
l=0.14; b=0.2;  rb = 2/3 * 0.5*((l/2)+(b/2));

% resize fy and allocate tz to speed up loop
fy=zeros(length(sy),length(camber)); tz=fy;

for i = 1:length(sy)
  for j = 1:length(camber) 
%   bore slip 
    sb = -rb*omega*sin(camber(j))/abs(vt);           
%   generalized slip (no longitudinal slip here)
    sg =  sqrt( sy(i)^2 + sb^2 );  
%   generalized tire force 
    [f, fos] = tmy_fcombined(sg, dfy0,fym,sym,fys,sys);  
%   lateral force (no longitudinal force here)
    if sg>0, fy(i,j) = f*sy(i)/sg; else fy(i,j)=0; end               
%   tire offset normalized to contact length l and self aligning torque
    nto = tmy_tireoff(sy(i), nto0,synto0,syntos);  ts = -fy(i,j)*nto*l;     
%   bore torque  +  resulting torque
    if sg>0, tb = rb*f*sb/sg; else tb=0; end,   tz(i,j) = ts + tb;
%   camber slip and force
    sy_c=-omega*sin(camber(j))/abs(vt)*l/2;  fy_c = fos/3*sy_c;    
%   add camber force 
    fy(i,j) = fy(i,j) + fy_c;    
  end
end
figure % open new figure
subplot(2,2,1), plot(atan(sy)*180/pi,fy'/1000),axis([-25,25,-4,4]), grid on
subplot(2,2,2), plot(atan(sy)*180/pi,tz'), axis([-25,25,-80,80]), grid on